#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Oct 24 15:18:55 2019

Tensorflow example solution for training a simple classification model for the breat cancer dataset

@author: Benjamin Milde
"""

# -*- coding: utf-8 -*-
import tensorflow as tf
import numpy as np

from sklearn.datasets import load_breast_cancer
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler

import pylab as pl

# Try running without a seed, and your solution will be slightly different each time:
np.random.seed(42)

# Select the precision of the model:

# This is standard precision (32 bit)
type_precision = tf.float32

# Double precision:
# type_precision = tf.float64

# There is also float 16, 16 bit precision:
# type_precision = tf.float16

def load_data():
    ## load the data
    bc = load_breast_cancer()
    x_data = bc['data']                                 # shape: (569,30)
    y_data = bc['target'].reshape(len(bc['target']), 1) # shape: (569, 1)

    x_train, x_test, y_train, y_test = train_test_split(x_data, y_data)

    scaler = StandardScaler()
    scaler.fit(x_train)
    x_train = scaler.transform(x_train)
    x_test = scaler.transform(x_test)

    return x_train, y_train, x_test, y_test


def build_graph(learning_rate):
    
    input_dim = 30
    
    ## define the model
    x = tf.placeholder(dtype_precision, (None, input_dim))
    y = tf.placeholder(dtype_precision, (None,  1))

    w = tf.Variable(tf.ones([input_dim, 1], dtype=dtype_precision), name="weight", dtype=dtype_precision)
    b = tf.Variable(tf.zeros([1], dtype=dtype_precision), name="bias", dtype=dtype_precision)

    yhat = tf.add(tf.matmul(x, w), b)
    
    print(tf.shape(yhat))
    
    loss = tf.reduce_mean(tf.nn.sigmoid_cross_entropy_with_logits(labels=y, logits=yhat))

    ## define the optimizer
    optimizer = tf.train.GradientDescentOptimizer(learning_rate).minimize(loss)

    return x, y, tf.sigmoid(yhat), loss, optimizer

def main():

    ## Hyperparameters - you can try to tune these to get better results
    learning_rate = 0.1
    epochs = 100

    x_train, y_train, x_test, y_test = load_data()
    x, y, pred, loss, optimizer = build_graph(learning_rate)

    ## start the session
    with tf.Session() as sess:

        ## initalize parameters
        sess.run(tf.global_variables_initializer())

        train_dict = {x: x_train, y: y_train}
        test_dict  = {x: x_test,  y: y_test}

        # Initial loss should be very high, since we have not run any optimization step yet
        print("Initial training loss: " + str(sess.run(loss, train_dict)))

        test_losses = []
        train_losses = []

        for i in range(epochs):
            ## run the optimizer
            sess.run(optimizer, train_dict)

            ## compute train and test losses        
            train_loss = sess.run(loss, train_dict)
            test_loss = sess.run(loss, test_dict)
            
            # append to lists for plotting it later
            train_losses.append(train_loss)
            test_losses.append(test_loss)
            
        # plot train and test losses    
        pl.legend("train", "test")
        pl.plot(train_losses)
        pl.plot(test_losses)
        
        pl.show()

        print("Final test loss: " + str(sess.run(loss, test_dict)))
        
if __name__ == "__main__":
    main()